#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# insert the package for universal imports.
import os, sys, syst3m
sys.path.insert(1, syst3m.defaults.get_source_path(__file__, back=2))
from sshmount.classes.config import *
from sshmount.classes import *
from sshmount.classes.sshmount import sshmount

# the cli object class.
class CLI(cl1.CLI):
	def __init__(self):
		
		# defaults.
		cl1.CLI.__init__(self,
			modes={
				"--mount <alias>:<remote> <path>":"Mount a remote directory.",
				"--unmount <path>":"Unmount a mounted remote directory.",
				"--index <path> / <alias>:<remote>":"Index the specified path / alias:remote.",
				"--daemon <alias>:<remote> <path>":"Start a sshmount daemon.",
				"--stop-daemon <path>":"Stop a sshmount daemon.",
				"-h / --help":"Show the documentation.",
			},
			options={
				"-j / --json":"Print the response objects in json format.",
			},
			alias=ALIAS,
			executable=__file__,
		)

		#
	def start(self):

		# options.
		json = self.arguments_present(["-j", "--json"])
		
		# help.
		if self.arguments_present(['-h', '--help']):
			print(self.documentation)

		# mount.
		elif self.argument_present('--mount'):
			remote = self.get_argument("--mount", index=1)
			if ":" not in remte:
				r3sponse.log(error=f"Invalid <alias>:<remote> format: {remote}.")
				self.stop()
			alias,remote = remote.split(":")
			path = self.get_argument("--mount", index=2)
			response = sshmount.mount(
				alias=alias, 
				remote=remote, 
				path=path,
				delete=self.argument_present("--delete"),
				forced=self.argument_present("--forced"), )
			r3sponse.log(response=response, json=json)

		# unmount.
		elif self.argument_present('--unmount'):
			path = self.get_argument("--unmount", index=1)
			response = sshmount.unmount(
				path=path,
				forced=self.argument_present("--forced"), 
				sudo=self.argument_present("--sudo"), )
			r3sponse.log(response=response, json=json)

		# index.
		elif self.argument_present('--index'):
			index = self.get_argument("--index")
			if ":" in index:
				alias,remote = index.split(":")
				response = sshmount.index(path=remote, alias=alias)
			else:
				response = sshmount.index(path=index)
			r3sponse.log(response=response, json=json)

		# start daemon.
		elif self.argument_present('--daemon'):
			remote = self.get_argument("--daemon", index=1)
			if ":" not in remte:
				r3sponse.log(error=f"Invalid <alias>:<remote> format: {remote}.")
				self.stop()
			alias,remote = remote.split(":")
			path = self.get_argument("--daemon", index=2)
			response = sshmount.daemon(alias=alias, remote=remote, path=path)
			r3sponse.log(response=response, json=json)

		# stop daemon.
		elif self.argument_present('--stop-daemon'):
			path = self.get_argument("--stop-daemon", index=1)
			response = sshmount.stop_daemon(path)
			r3sponse.log(response=response, json=json)

		# invalid.
		else: self.invalid()

		#
	def invalid():
		print(self.documentation)
		print("Selected an invalid mode.")
		self.stop()
	def stop():
		sys.exit(1)
		#
	def get_passphrase(self, required=True):
		return self.get_argument("--passphrase", required=required).replace("\\", "").replace("\ ", "")
# main.
if __name__ == "__main__":
	cli = CLI()
	cli.start()
