#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# imports.
from sshmount.classes.config import *
from sshmount.classes import utils

# universal functions.
def set_status(path=None, status=None):
	status_path = f"{HOME}/.sshmount/daemons/"+path.replace("/", "\\")
	Files.save(status_path, status, format="str")
def get_status(path):
	status_path = f"{HOME}/.sshmount/daemons/"+path.replace("/", "\\")
	try:
		return Files.load(status_path, format="str")
	except FileNotFoundError:
		set_status(path, "none")
		return Files.load(status_path, format="str")
def pull(
	path=None, 
	alias=None, 
	remote=None, 
	exclude=['__pycache__', '.DS_Store'],
	delete=True,
):
	# pull.
	exclude_str = ""
	for i in exclude: exclude_str += f"--exclude {i} "
	delete_str = utils.convert_bool(delete, true="--delete", false="")
	command = f"rsync -azP {alias}:{remote} {path} {exclude_str}{delete_str}"
	output = syst3m.utils.__execute_script__(command)
	set_status(path, "*mounted*")
	# handler.
	return r3sponse.success_response(f"Successfully pulled [{alias}:{remote}] to [{path}].")
def push(
	path=None, 
	alias=None, 
	remote=None, 
	exclude=['__pycache__', '.DS_Store'],
	delete=True,
):
	# push.
	exclude_str = ""
	for i in exclude: exclude_str += f"--exclude {i} "
	delete_str = utils.convert_bool(delete, true="--delete", false="")
	command = f"rsync -azP {path} {alias}:{remote} {exclude_str}{delete_str}"
	output = syst3m.utils.__execute_script__(command)
	# handler.
	return r3sponse.success_response(f"Successfully pushed [{path}] to [{alias}:{remote}].")

# the sshmount object class.
class SSHMount(object):
	def __init__(self):
		a=1
	def mount(self, 
		# the local path.
		path=None, 
		# the remote path.
		remote=None, 
		# ssh alias.
		alias=None,
		# delete.
		delete=False,
		# forced.
		forced=False,
		# exclude.
		exclude=['__pycache__', '.DS_Store'],
		# accept new host verification keys.
		accept_new_host_keys=True,
	):

		# checks.
		response = r3sponse.check_parameters({
			"path:str":path,
			"remote:str":remote,
			"alias:str":alias,
			"exclude:list":exclude,
			"delete:bool":delete,
			"forced:bool":forced,
			"accept_new_host_keys:bool":accept_new_host_keys, })
		if not response.success: return response
		elif os.path.exists(path):
			return r3sponse.error_response(f"Path [{path}] already exists.")
		elif "*mounted*" in get_status(path):
			return r3sponse.error_response(f"Path [{path}] is already mounted.")

		# tests.
		response = utils.test_ssh(alias=alias, accept_new_host_keys=accept_new_host_keys)
		if not response.success: return response
		response = utils.test_sshmount(alias=alias, accept_new_host_keys=accept_new_host_keys)
		if not response.success: return response
		response = utils.test_ssh_path(alias=alias, path=path, accept_new_host_keys=accept_new_host_keys)
		if not response.success: return response

		# pull.
		response = pull(
			path=path, 
			alias=alias, 
			remote=remote, 
			exclude=exclude,
			delete=delete,)
		if not r3sponse.success: return response

		# start daemon.
		p = subprocess.Popen([SOURCE_PATH, "--daemon", f"{alias}:{remote}", path])
		p.start()

		# handler.
		return r3sponse.success_response(f"Successfully mounted [{alias}:{remote}] ==> [{path}].", {
			"index":index,
		})

		#
	def unmount(self, 
		# the local path.
		path=None, 
		# forced required.
		forced=False,
		# sudo required.
		sudo=False,
	):

		# checks.
		response = r3sponse.check_parameters({
			"path:str":path,
			"forced:bool":forced,
			"sudo:bool":sudo, })
		if not response.success: return response
		elif not os.path.exists(path):
			return r3sponse.error_response(f"Path [{path}] does not exist.")
		elif not os.path.isdir(path):
			return r3sponse.error_response(f"Path [{path}] is not a directory.")
		elif "*mounted*" not in get_status(path):
			return r3sponse.error_response(f"Path [{path}] is not mounted.")

		# wait for daemon stop.
		response = self.stop_daemon(path=path)
		if not success: return response

		# delete local.
		"""
		if sudo: sudo_str = "sudo "
		else: sudo_str = ""
		os.system(f"{sudo_str}rm -fr {path}")
		if os.path.exists(path):
			return r3sponse.error_response(f"Failed to unmount [{path}].")
		"""

		# handler.
		return r3sponse.success_response(f"Successfully unmounted [{path}].")

		#
	def index(self, path=None, alias=None):

		# checks.
		if path == None:
			return r3sponse.error_response(f"Define parameter: path.")

		# remote.
		if alias != None:
			output = syst3m.utils.__execute_script__(f"ssh {alias} 'python3 /usr/local/lib/sshmount --index {path} ' ")
			try: response = json.loads(output)
			except Exception as e: 
				return r3sponse.error_response(f"Failed to serialize remote {alias} output: {output}")
			return r3sponse.ResponseObject(response)

		# local.
		else:
			if not os.path.exists(path):
				return r3sponse.error_response(f"Path [{path}] does not exist.")
			elif not os.path.isdir(path):
				return r3sponse.error_response(f"Path [{path}] is not a directory.")

			# index.
			index = {}
			for path in Files.Directory(path=path).paths(recursive=True):
				index[path] = Formats.FilePath(path).mtime(format="seconds")

			# handler.
			return r3sponse.success_response(f"Successfully indexed {len(index)} files from directory [{path}].", {
				"index":index,
			})

			#
	# the mounted daemon.
	def daemon(self, 
		# the ssh alias.
		alias=None, 
		# the remote path.
		remote=None, 
		# thel local path.
		path=None, 
		# settings.
		start=True,
	):
		daemon = self.Daemon({
			"alias":alias,
			"remote":remote,
			"path":path,
		})
		if start: daemon.start()
		return daemon
	def stop_daemon(self, path, timeout=60, sleeptime=1):
		set_status(path, "*stop*")
		stopped = False
		for i in range(timeout/sleeptime):
			status = get_status(path)
			if "*stopped*" in status or "*crashed*" in status:
				stopped = True
				break
			time.sleep(sleeptime)
		if stopped:
			return r3sponse.success_response(f"Successfully stopped sshmount daemon [{path}].")
		else:
			return r3sponse.error_response(f"Failed to stop sshmount daemon [{path}].")
	class Daemon(syst3m.objects.Thread):
		def __init__(self, attributes={}):
			syst3m.objects.Thread.__init__(self)
			self.assign(attributes)
			self.id = f"{self.alias}:{self.remote} ==> {self.path}"
		def run(self):

			# checks.
			if not os.path.exists(self.path):
				set_status(self.path, "*crashed*")
				raise ValueError(f"SSHMount daemon ({self.id}): Path [{self.path}] does not exist.")
			elif not os.path.isdir(self.path):
				set_status(self.path, "*crashed*")
				raise ValueError(f"SSHMount daemon ({self.id}): Path [{self.path}] is not a directory.")
			elif "*mounted*" not in get_status(path):
				set_status(self.path, "*crashed*")
				raise ValueError(f"SSHMount daemon ({self.id}): Path [{self.path}] is not mounted.")

			# start.
			while True:

				# check stop command.
				if "*stop*" in get_status(self.path):
					break

				# if file no longer exists stop daemon.
				if not os.path.exists(self.path):
					raise ValueError(f"SSHMount daemon ({self.id}): Mounted directory [{self.path}] from [{self.alias}:{self.remote}] no longer exists.")

			# stop.
			set_status(self.path, "*stopped*")

# initialized objects.
sshmount = SSHMount()

print(sshmount.mount(
	alias="dev.vandenberghinc.com",
	remote="/tmp/testmount",
	path="/tmp/testmount",))
quit()